@@ -18,6 +18,8 @@
 #  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 
+import re
+
 import numpy as np
 from numpy.testing import assert_allclose
 
@@ -33,10 +35,10 @@ from sherpa.data import Data1D, Data1DInt
 from sherpa.astro import ui
 
 from sherpa.utils import linear_interp
-from sherpa.utils.err import ModelErr
+from sherpa.utils.err import DataErr, ModelErr
 
 from sherpa.models.regrid import ModelDomainRegridder1D, EvaluationSpace1D, \
-    EvaluationSpace2D
+    EvaluationSpace2D, PointAxis, IntegratedAxis
 
 
 @pytest.fixture(params=[True, False])
@@ -413,12 +415,11 @@ def test_regrid1d_error_calc_no_args(setup_1d):
     rmdl = ModelDomainRegridder1D(grid_evaluate)
     mdl = rmdl.apply_to(internal_mdl)
 
-    with pytest.raises(ModelErr) as excinfo:
+    with pytest.raises(ModelErr,
+                       match=ModelErr.dict['nogrid']):
         pvals = [p.val for p in internal_mdl.pars]
         mdl.calc(p=pvals)
 
-    assert ModelErr.dict['nogrid'] in str(excinfo.value)
-
 
 def test_regrid1d_error_grid_mismatch_1(setup_1d):
     """Internal grid is integrated but given points"""
@@ -431,11 +432,10 @@ def test_regrid1d_error_grid_mismatch_1(setup_1d):
 
     mdl = rmdl.apply_to(internal_mdl)
     grid_run = np.arange(0, 20, 10)
-    with pytest.raises(ModelErr) as excinfo:
+    with pytest.raises(ModelErr,
+                       match=re.escape(ModelErr.dict['needsint'])):
         mdl(grid_run)
 
-    assert ModelErr.dict['needsint'] in str(excinfo.value)
-
 
 def test_ui_regrid1d_non_overlapping_not_allowed():
     """Integrated data space must not overlap"""
@@ -448,11 +448,10 @@ def test_ui_regrid1d_non_overlapping_not_allowed():
     b1.ampl.max = 100
     grid_hi = np.linspace(2, 101, 600)
     grid_lo = np.linspace(1, 100, 600)
-    with pytest.raises(ModelErr) as excinfo:
+    with pytest.raises(ModelErr,
+                       match=re.escape(ModelErr.dict['needsint'])):
         b1.regrid(grid_lo, grid_hi)
 
-    assert ModelErr.dict['needsint'] in str(excinfo.value)
-
 
 def test_low_level_regrid1d_non_overlapping_not_allowed():
     """Integrated data space must not overlap"""
@@ -460,11 +459,10 @@ def test_low_level_regrid1d_non_overlapping_not_allowed():
     c = Box1D()
     lo = np.linspace(1, 100, 600)
     hi = np.linspace(2, 101, 600)
-    with pytest.raises(ModelErr) as excinfo:
+    with pytest.raises(ModelErr,
+                       match=re.escape(ModelErr.dict['needsint'])):
         c.regrid(lo, hi)
 
-    assert ModelErr.dict['needsint'] in str(excinfo.value)
-
 
 def test_regrid1d_error_grid_mismatch_2(setup_1d):
     """Internal grid is points but given integrated"""
@@ -476,11 +474,10 @@ def test_regrid1d_error_grid_mismatch_2(setup_1d):
 
     mdl = rmdl.apply_to(internal_mdl)
     grid_run = np.arange(0, 20, 10)
-    with pytest.raises(ModelErr) as excinfo:
+    with pytest.raises(ModelErr,
+                       match=ModelErr.dict['needspoint']):
         mdl(grid_run[:-1], grid_run[1:])
 
-    assert ModelErr.dict['needspoint'] in str(excinfo.value)
-
 
 @pytest.mark.parametrize("requested",
                          [np.arange(1, 7, 0.1),
@@ -964,10 +961,10 @@ def test_regrid1d_int_flux():
 
 
 @pytest.mark.parametrize('x, y', [
-    (None, [1, 2]),
-    ([1, 2], None),
     (None, None),
-    ([], [1, 2]),
+    (None, [2, 3, 4]),
+    ([], [2, 3, 4]),
+    ([1, 2], None),
     ([1, 2], [])
 ])
 def test_evaluation_space2d_empty(x, y):
@@ -978,11 +975,36 @@ def test_evaluation_space2d_empty_no_args():
     assert EvaluationSpace2D().is_empty
 
 
+@pytest.mark.parametrize("xaxis", [True, False])
+@pytest.mark.parametrize("lo,hi", [([], [2, 3]),
+                                   (None, [2, 3]),
+                                   ([1], [2, 3]),
+                                   ([1, 2], []),
+                                   ([1, 2], None),
+                                   ([1, 2], [1])
+                                   ])
+def test_evaluation_space2d_check_axes_are_integrated(xaxis, lo, hi):
+    """Check these calls fail.
+
+    Prior to 4.14.1 you could set the integrated axes to
+    different sizes.
+    """
+
+    olo = [1, 2]
+    ohi = [2, 3]
+    if xaxis:
+        args = {"x": lo, "xhi": hi, "y": olo, "yhi": ohi}
+    else:
+        args = {"x": olo, "xhi": ohi, "y": lo, "yhi": hi}
+
+    with pytest.raises(DataErr,
+                       match=r"^size mismatch between lo and hi: (\d|None) vs (\d|None)$"):
+        EvaluationSpace2D(**args)
+
+
 @pytest.mark.parametrize('xlo, xhi, ylo, yhi, is_integrated', [
-    ([1, 2], [2, 3], [1, 2], [2, 3], True),
-    ([1, 2], [2, 3], None, None, False),
-    ([1, 2], [2, 3],  [], [2, 3], False),
-    ([1, 2], [2, 3], [1, 2], [], False),
+    ([1, 2], [2, 3], [1, 2, 3], [2, 3, 4], True),
+    ([1, 2], None, [1, 2], None, False),
 ])
 def test_evaluation_space2d_is_integrated(xlo, xhi, ylo, yhi, is_integrated):
     assert EvaluationSpace2D(x=xlo, xhi=xhi, y=ylo, yhi=yhi).is_integrated\
@@ -1005,7 +1027,8 @@ def test_evaluation_space2d_is_ascending(xlo, xhi, ylo, yhi, is_ascending):
 
 
 def test_evaluation_space2d_is_ascending_error():
-    with pytest.raises(ValueError):
+    with pytest.raises(DataErr,
+                       match="Axis is empty or has a size of 0"):
         EvaluationSpace2D(x=None, xhi=None, y=None, yhi=None).is_ascending
 
 
@@ -1090,16 +1113,86 @@ def test_wrong_kwargs():
     d = Data1D('tst', xgrid, np.ones_like(xgrid))
     mdl = Box1D()
     requested = np.arange(1, 7, 0.1)
-    with pytest.raises(TypeError) as excinfo:
+    with pytest.raises(TypeError,
+                       match="unknown keyword argument: 'fubar'"):
         d.eval_model(mdl.regrid(requested, fubar='wrong_kwargs'))
 
-    assert "unknown keyword argument: 'fubar'" in str(excinfo.value)
-
 
 def test_interp_method_is_callable():
     """Check that method is callable."""
     rmdl = ModelDomainRegridder1D()
-    with pytest.raises(TypeError) as exc:
+    with pytest.raises(TypeError,
+                       match="method argument 'True' is not callable"):
         rmdl.method = True
 
-    assert str(exc.value) == "method argument 'True' is not callable"
+
+def test_axis_check_not_a_scalar():
+    """Mainly to ensure this error condition is checked"""
+
+    with pytest.raises(DataErr,
+                       match="Array must be a sequence or None"):
+        PointAxis(23)
+
+
+def test_axis_check_not_multidim():
+    """Mainly to ensure this error condition is checked"""
+
+    with pytest.raises(DataErr,
+                       match="Array must be 1D"):
+        PointAxis(np.arange(12).reshape(3, 4))
+
+
+def test_pointaxis_not_integrated():
+    """Check for the empty case"""
+
+    assert not PointAxis([]).is_integrated
+
+
+def test_integratedaxis_is_integrated():
+    """Check for the empty case"""
+
+    assert IntegratedAxis([], []).is_integrated
+
+
+def test_pointaxis_size():
+    """Pick a descending axis for fun"""
+    assert PointAxis([4, 3, 2]).size == 3
+
+
+def test_integratedaxis_size():
+    assert IntegratedAxis([4, 3, 2], [4.5, 3.5, 3]).size == 3
+
+
+def test_pointaxis_start():
+    """Pick a descending axis for fun"""
+    assert PointAxis([4, 3, 2]).start == 2
+
+
+def test_pointaxis_end():
+    """Pick a descending axis for fun"""
+    assert PointAxis([4, 3, 2]).end == 4
+
+
+def test_integratedaxis_start():
+    """Pick a descending axis for fun"""
+    assert IntegratedAxis([4, 3, 2], [4.5, 3.5, 3]).start == 2
+
+
+def test_integratedaxis_end():
+    """Pick a descending axis for fun"""
+    assert IntegratedAxis([4, 3, 2], [4.5, 3.5, 3]).end == pytest.approx(4.5)
+
+
+def test_evaluationspace1d_zeros_like_empty():
+
+    assert EvaluationSpace1D().zeros_like() == pytest.approx([])
+
+
+def test_evaluationspace1d_zeros_like_point():
+
+    assert EvaluationSpace1D([3, 2, -1]).zeros_like() == pytest.approx([0, 0, 0])
+
+
+def test_evaluationspace1d_zeros_like_integrated():
+
+    assert EvaluationSpace1D([3, 2, -1], [2, 1, -3]).zeros_like() == pytest.approx([0, 0, 0])
