@@ -450,13 +450,11 @@ def test_fit_raises_error_on_bgsubtraction(stat):
     fit = setup_pha_single(False, False, False, None, None, stat=statobj)
     fit.data.subtract()
 
-    with pytest.raises(FitErr) as excinfo:
+    with pytest.raises(FitErr,
+                       match=f"{statobj.name} statistics cannot be used with " +
+                       "background subtracted data"):
         fit.fit()
 
-    emsg = f"{statobj.name} statistics cannot be used with " + \
-           "background subtracted data"
-    assert str(excinfo.value) == emsg
-
 
 expected_leastsq = 31.5625
 
@@ -1751,12 +1749,10 @@ def test_fit_single_fails_nobins(stat):
     statobj = stat()
     fit = setup_stat_single(statobj, True, False)
     fit.data.ignore(None, None)
-    with pytest.raises(DataErr) as excinfo:
+    with pytest.raises(DataErr,
+                       match="mask excludes all data"):
         fit.fit()
 
-    emsg = 'mask excludes all data'
-    assert str(excinfo.value) == emsg
-
 
 @pytest.mark.parametrize("stat",
                          [LeastSq, Chi2, Chi2Gehrels, Chi2DataVar,
@@ -1775,12 +1771,10 @@ def test_fit_single_fails_pha_nobins(stat):
     statobj = stat()
     fit = setup_pha_single(True, True, False, None, None, stat=statobj)
     fit.data.ignore(None, None)
-    with pytest.raises(DataErr) as excinfo:
+    with pytest.raises(DataErr,
+                       match="mask excludes all data"):
         fit.fit()
 
-    emsg = 'mask excludes all data'
-    assert str(excinfo.value) == emsg
-
 
 # Note that prior to Sherpa 4.8.1 the error here would have been a
 # ValueError with the message 'calculation of errors has failed
@@ -1798,13 +1792,11 @@ def test_fit_single_fails_dvar(usesys):
 
     statobj = Chi2DataVar()
     fit = setup_stat_single(statobj, False, usesys)
-    with pytest.raises(FitErr) as excinfo:
+    with pytest.raises(FitErr,
+                       match="zeros found in uncertainties, consider using " +
+                       "calculated uncertainties"):
         fit.fit()
 
-    emsg = 'zeros found in uncertainties, consider using ' + \
-           'calculated uncertainties'
-    assert str(excinfo.value) == emsg
-
 
 # Calculated using LevMar
 #
@@ -2135,12 +2127,10 @@ def test_est_errors_single_fails_lsq(method, estmethod, usestat, usesys):
     fit.estmethod = estmethod()
     fit.fit()
 
-    with pytest.raises(EstErr) as excinfo:
+    with pytest.raises(EstErr,
+                       match="cannot estimate confidence limits with LeastSq"):
         fit.est_errors()
 
-    emsg = 'cannot estimate confidence limits with LeastSq'
-    assert str(excinfo.value) == emsg
-
 
 @pytest.mark.parametrize("method, estmethod", [
     (LevMar, Covariance),
@@ -2163,12 +2153,10 @@ def test_est_errors_single_fails_nobins(method, estmethod):
     fit.fit()
 
     fit.data.ignore(None, None)
-    with pytest.raises(DataErr) as excinfo:
+    with pytest.raises(DataErr,
+                       match="mask excludes all data"):
         fit.est_errors()
 
-    emsg = 'mask excludes all data'
-    assert str(excinfo.value) == emsg
-
 
 # Using Covariance does not trigger this error.
 @pytest.mark.parametrize("stat, method, estmethod", [
@@ -2195,12 +2183,10 @@ def test_est_errors_single_fails_badfit(stat, method, estmethod):
     fit.method = method()
     fit.estmethod = estmethod()
 
-    with pytest.raises(EstErr) as excinfo:
+    with pytest.raises(EstErr,
+                       match="reduced statistic larger than 3"):
         fit.est_errors()
 
-    emsg = 'reduced statistic larger than 3'
-    assert str(excinfo.value) == emsg
-
 
 # Restrict the choices run here to save time; ideally the parameter
 # values should be set rather than rely on a call to fit.
@@ -2641,11 +2627,10 @@ def test_563_still_exists():
     mdl.c0 = 5.1
 
     fit = Fit(d, mdl, stat=WStat())
-    with pytest.raises(AttributeError) as excinfo:
+    with pytest.raises(AttributeError,
+                       match="'list' object has no attribute 'size'"):
         fit.calc_stat_info()
 
-    assert "'list' object has no attribute 'size'" in str(excinfo.value)
-
 
 # Note that the dof=1 tests don't really add any coverage to other
 # tests, since they are no different than dof>>1, but are included so that
@@ -2826,9 +2811,9 @@ def make_data(data_class):
     assert False  # programmer error
 
 
-@pytest.mark.parametrize("data_class,model_class",
-                         [(Data1D, Const2D), (Data2D, Const1D)])
-def test_fit_ensures_data_and_model_dimensionality_matches(data_class, model_class):
+@pytest.mark.parametrize("data_class,ddim,model_class,mdim",
+                         [(Data1D, 1, Const2D, 2), (Data2D, 2, Const1D, 1)])
+def test_fit_ensures_data_and_model_dimensionality_matches(data_class, ddim, model_class, mdim):
     """Check that we error out when the dimensionality does not match.
 
     This does not catch the combined data or model (DataSimulFit and
@@ -2838,5 +2823,6 @@ def test_fit_ensures_data_and_model_dimensionality_matches(data_class, model_cla
 
     data = make_data(data_class)
     model = model_class()
-    # does not raise an error
-    Fit(data, model)
+    with pytest.raises(DataErr,
+                       match=f"Data and model dimensionality do not match: {ddim}D and {mdim}D"):
+        Fit(data, model)
