@@ -1,5 +1,5 @@
 #
-#  Copyright (C) 2019, 2020, 2021
+#  Copyright (C) 2019, 2020, 2021, 2022
 #  Smithsonian Astrophysical Observatory
 #
 #
@@ -38,15 +38,9 @@ from sherpa.astro.ui.utils import Session as AstroSession
 
 from sherpa.data import Data1D, Data1DInt, Data2D
 from sherpa.models import basic
-from sherpa.plot import CDFPlot, DataPlot, FitPlot, ModelPlot, \
-    PDFPlot, PSFPlot, PSFKernelPlot, ScatterPlot, TracePlot,\
-    DataContour, ModelContour, SourceContour, ResidContour, \
-    RatioContour, FitContour, LRHistogram, \
-    ModelHistogramPlot, ResidPlot, RatioPlot, DelchiPlot, ChisqrPlot, \
-    DataHistogramPlot
-
+import sherpa.plot
 from sherpa.stats import Chi2Gehrels
-from sherpa.utils.err import ArgumentErr, ArgumentTypeErr, IdentifierErr
+from sherpa.utils.err import ArgumentTypeErr, IdentifierErr, PlotErr
 from sherpa.utils.testing import requires_plotting, requires_pylab
 
 
@@ -116,9 +110,9 @@ def test_get_fit_plot(idval, clean_ui):
     else:
         f = ui.get_fit_plot(idval)
 
-    assert isinstance(f, FitPlot)
-    assert isinstance(f.dataplot, DataPlot)
-    assert isinstance(f.modelplot, ModelPlot)
+    assert isinstance(f, sherpa.plot.FitPlot)
+    assert isinstance(f.dataplot, sherpa.plot.DataPlot)
+    assert isinstance(f.modelplot, sherpa.plot.ModelPlot)
 
     dp = f.dataplot
     mp = f.modelplot
@@ -233,10 +227,10 @@ def change_fit(idval):
     change_model(idval)
 
 
-def check_example(xlabel='x'):
+def check_example(idval, xlabel='x'):
     """Check that the data plot has not changed"""
 
-    dplot = ui._session._dataplot
+    dplot = ui.get_data_plot(id=idval, recalc=False)
 
     assert dplot.xlabel == xlabel
     assert dplot.ylabel == 'y'
@@ -249,13 +243,13 @@ def check_example(xlabel='x'):
     assert dplot.yerr == pytest.approx(calc_errors(_data_y))
 
 
-def check_example_changed(xlabel='x'):
+def check_example_changed(idval, xlabel='x'):
     """Check that the data plot has changed
 
     Assumes change_example has been called
     """
 
-    dplot = ui._session._dataplot
+    dplot = ui.get_data_plot(id=idval, recalc=False)
 
     assert dplot.xlabel == xlabel
     assert dplot.ylabel == 'y'
@@ -280,45 +274,44 @@ def check_model_plot(plot, title='Model', xlabel='x', modelval=35):
     assert plot.yerr is None
 
 
-def check_model(xlabel='x'):
+def check_model(idval, xlabel='x'):
     """Check that the model plot has not changed"""
 
-    check_model_plot(ui._session._modelplot,
-                     title='Model', xlabel=xlabel)
+    mplot = ui.get_model_plot(id=idval, recalc=False)
+    check_model_plot(mplot, title='Model', xlabel=xlabel)
 
 
-def check_model_changed(xlabel='x'):
+def check_model_changed(idval, xlabel='x'):
     """Check that the model plot has changed
 
     Assumes change_model has been called
     """
 
-    check_model_plot(ui._session._modelplot,
-                     title='Model', xlabel=xlabel,
-                     modelval=41)
+    mplot = ui.get_model_plot(id=idval, recalc=False)
+    check_model_plot(mplot, title='Model', xlabel=xlabel, modelval=41)
 
 
-def check_source():
+def check_source(idval):
     """Check that the source plot has not changed"""
 
-    check_model_plot(ui._session._sourceplot,
-                     title='Source')
+    splot = ui.get_source_plot(id=idval, recalc=False)
+    check_model_plot(splot, title='Source')
 
 
-def check_source_changed():
+def check_source_changed(idval):
     """Check that the source plot has changed
 
     Assumes change_model has been called
     """
 
-    check_model_plot(ui._session._sourceplot,
-                     title='Source', modelval=41)
+    splot = ui.get_source_plot(id=idval, recalc=False)
+    check_model_plot(splot, title='Source', modelval=41)
 
 
-def check_resid(title='Residuals for example'):
+def check_resid(idval, title='Residuals for example'):
     """Check that the resid plot has not changed"""
 
-    rplot = ui._session._residplot
+    rplot = ui.get_resid_plot(id=idval, recalc=False)
     assert rplot.xlabel == 'x'
     assert rplot.ylabel == 'Data - Model'
     assert rplot.title == title
@@ -328,13 +321,13 @@ def check_resid(title='Residuals for example'):
     assert rplot.yerr == pytest.approx(calc_errors(_data_y))
 
 
-def check_resid_changed(title='Residuals for example'):
+def check_resid_changed(idval, title='Residuals for example'):
     """Check that the resid plot has changed
 
     Assumes that change_model has been called
     """
 
-    rplot = ui._session._residplot
+    rplot = ui.get_resid_plot(id=idval, recalc=False)
     assert rplot.xlabel == 'x'
     assert rplot.ylabel == 'Data - Model'
     assert rplot.title == title
@@ -344,13 +337,13 @@ def check_resid_changed(title='Residuals for example'):
     assert rplot.yerr == pytest.approx(calc_errors(_data_y))
 
 
-def check_resid_changed2(title='Residuals for example'):
+def check_resid_changed2(idval, title='Residuals for example'):
     """Check that the resid plot has changed
 
     Assumes that change_example and change_model has been called
     """
 
-    rplot = ui._session._residplot
+    rplot = ui.get_resid_plot(id=idval, recalc=False)
     assert rplot.xlabel == 'x'
     assert rplot.ylabel == 'Data - Model'
     assert rplot.title == title
@@ -360,10 +353,10 @@ def check_resid_changed2(title='Residuals for example'):
     assert rplot.yerr == pytest.approx(calc_errors(_data_y2))
 
 
-def check_ratio(title='Ratio of Data to Model for example'):
+def check_ratio(idval, title='Ratio of Data to Model for example'):
     """Check that the ratio plot has not changed"""
 
-    rplot = ui._session._ratioplot
+    rplot = ui.get_ratio_plot(id=idval, recalc=False)
     assert rplot.xlabel == 'x'
     assert rplot.ylabel == 'Data / Model'
     assert rplot.title == title
@@ -374,13 +367,13 @@ def check_ratio(title='Ratio of Data to Model for example'):
     assert rplot.yerr == pytest.approx(dy)
 
 
-def check_ratio_changed():
+def check_ratio_changed(idval):
     """Check that the ratio plot has changed
 
     Assumes that change_example has been called
     """
 
-    rplot = ui._session._ratioplot
+    rplot = ui.get_ratio_plot(id=idval, recalc=False)
     assert rplot.xlabel == 'x'
     assert rplot.ylabel == 'Data / Model'
     assert rplot.title == 'Ratio of Data to Model for example'
@@ -391,13 +384,13 @@ def check_ratio_changed():
     assert rplot.yerr == pytest.approx(dy)
 
 
-def check_ratio_changed2(title='Ratio of Data to Model for example'):
+def check_ratio_changed2(idval, title='Ratio of Data to Model for example'):
     """Check that the ratio plot has changed
 
     Assumes that change_example and change_model has been called
     """
 
-    rplot = ui._session._ratioplot
+    rplot = ui.get_ratio_plot(id=idval, recalc=False)
     assert rplot.xlabel == 'x'
     assert rplot.ylabel == 'Data / Model'
     assert rplot.title == title
@@ -408,10 +401,10 @@ def check_ratio_changed2(title='Ratio of Data to Model for example'):
     assert rplot.yerr == pytest.approx(dy)
 
 
-def check_delchi(title='Sigma Residuals for example'):
+def check_delchi(idval, title='Sigma Residuals for example'):
     """Check that the delchi plot has not changed"""
 
-    rplot = ui._session._delchiplot
+    rplot = ui.get_delchi_plot(id=idval, recalc=False)
     assert rplot.xlabel == 'x'
     assert rplot.ylabel == 'Sigma'
     assert rplot.title == title
@@ -424,13 +417,13 @@ def check_delchi(title='Sigma Residuals for example'):
     assert rplot.yerr == pytest.approx([1.0 for y in _data_y])
 
 
-def check_delchi_changed(title='Sigma Residuals for example'):
+def check_delchi_changed(idval, title='Sigma Residuals for example'):
     """Check that the delchi plot has changed
 
     Assumes that change_example has been called
     """
 
-    rplot = ui._session._delchiplot
+    rplot = ui.get_delchi_plot(id=idval, recalc=False)
     assert rplot.xlabel == 'x'
     assert rplot.ylabel == 'Sigma'
     assert rplot.title == title
@@ -443,13 +436,13 @@ def check_delchi_changed(title='Sigma Residuals for example'):
     assert rplot.yerr == pytest.approx([1.0 for y in _data_x])
 
 
-def check_delchi_changed2(title='Sigma Residuals for example'):
+def check_delchi_changed2(idval, title='Sigma Residuals for example'):
     """Check that the delchi plot has changed
 
     Assumes that change_example and change_model has been called
     """
 
-    rplot = ui._session._delchiplot
+    rplot = ui.get_delchi_plot(id=idval, recalc=False)
     assert rplot.xlabel == 'x'
     assert rplot.ylabel == 'Sigma'
     assert rplot.title == title
@@ -462,10 +455,10 @@ def check_delchi_changed2(title='Sigma Residuals for example'):
     assert rplot.yerr == pytest.approx([1.0 for y in _data_x])
 
 
-def check_chisqr():
+def check_chisqr(idval):
     """Check that the chisqr plot has not changed"""
 
-    rplot = ui._session._chisqrplot
+    rplot = ui.get_chisqr_plot(id=idval, recalc=False)
     assert rplot.xlabel == 'x'
     assert rplot.ylabel == '$\\chi^2$'
     assert rplot.title == '$\\chi^2$ for example'
@@ -478,13 +471,13 @@ def check_chisqr():
     assert rplot.yerr is None
 
 
-def check_chisqr_changed():
+def check_chisqr_changed(idval):
     """Check that the chisqr plot has changed
 
     Assumes that change_example has been called
     """
 
-    rplot = ui._session._chisqrplot
+    rplot = ui.get_chisqr_plot(id=idval, recalc=False)
     assert rplot.xlabel == 'x'
     assert rplot.ylabel == '$\\chi^2$'
     assert rplot.title == '$\\chi^2$ for example'
@@ -497,78 +490,78 @@ def check_chisqr_changed():
     assert rplot.yerr is None
 
 
-def check_fit():
+def check_fit(idval):
     """Check that the fit plot has not changed"""
 
-    check_example()
-    check_model()
+    check_example(idval)
+    check_model(idval)
 
 
-def check_fit_changed():
+def check_fit_changed(idval):
     """Check that the fit plot has changed
 
     Assumes that change_fit has been called
     """
 
-    check_example_changed()
-    check_model_changed()
+    check_example_changed(idval)
+    check_model_changed(idval)
 
 
-def check_fit_resid():
+def check_fit_resid(idval):
     """Check that the fit + resid plot has not changed"""
 
-    check_example(xlabel='')
-    check_model(xlabel='')
-    check_resid(title='')
+    check_example(idval, xlabel='')
+    check_model(idval, xlabel='')
+    check_resid(idval, title='')
 
 
-def check_fit_resid_changed():
+def check_fit_resid_changed(idval):
     """Check that the fit + resid plot has changed
 
     Assumes that change_fit has been called
     """
 
-    check_example_changed(xlabel='')
-    check_model_changed(xlabel='')
-    check_resid_changed2(title='')
+    check_example_changed(idval, xlabel='')
+    check_model_changed(idval, xlabel='')
+    check_resid_changed2(idval, title='')
 
 
-def check_fit_ratio():
+def check_fit_ratio(idval):
     """Check that the fit + ratio plot has not changed"""
 
-    check_example(xlabel='')
-    check_model(xlabel='')
-    check_ratio(title='')
+    check_example(idval, xlabel='')
+    check_model(idval, xlabel='')
+    check_ratio(idval, title='')
 
 
-def check_fit_ratio_changed():
+def check_fit_ratio_changed(idval):
     """Check that the fit + ratio plot has changed
 
     Assumes that change_fit has been called
     """
 
-    check_example_changed(xlabel='')
-    check_model_changed(xlabel='')
-    check_ratio_changed2(title='')
+    check_example_changed(idval, xlabel='')
+    check_model_changed(idval, xlabel='')
+    check_ratio_changed2(idval, title='')
 
 
-def check_fit_delchi():
+def check_fit_delchi(idval):
     """Check that the fit + delchi plot has not changed"""
 
-    check_example(xlabel='')
-    check_model(xlabel='')
-    check_delchi(title='')
+    check_example(idval, xlabel='')
+    check_model(idval, xlabel='')
+    check_delchi(idval, title='')
 
 
-def check_fit_delchi_changed():
+def check_fit_delchi_changed(idval):
     """Check that the fit + delchi plot has changed
 
     Assumes that change_fit has been called
     """
 
-    check_example_changed(xlabel='')
-    check_model_changed(xlabel='')
-    check_delchi_changed2(title='')
+    check_example_changed(idval, xlabel='')
+    check_model_changed(idval, xlabel='')
+    check_delchi_changed2(idval, title='')
 
 
 _plot_all = [
@@ -634,7 +627,7 @@ def test_plot_xxx(idval, pfunc, checkfunc, clean_ui):
     else:
         pfunc(idval)
 
-    checkfunc()
+    checkfunc(idval)
 
 
 @requires_plotting
@@ -678,7 +671,7 @@ def test_plot_xxx_replot(idval, plotfunc, changefunc, checkfunc, clean_ui):
     else:
         plotfunc(idval, replot=True)
 
-    checkfunc()
+    checkfunc(idval)
 
 
 @requires_plotting
@@ -725,31 +718,31 @@ def test_plot_xxx_change(idval, plotfunc, changefunc, checkfunc, clean_ui):
     else:
         plotfunc(idval)
 
-    checkfunc()
+    checkfunc(idval)
 
 
-_dplot = (ui.get_data_plot_prefs, "_dataplot", ui.plot_data)
-_mplot = (ui.get_model_plot_prefs, "_modelplot", ui.plot_model)
+_dplot = (ui.get_data_plot_prefs, ui.get_data_plot, ui.plot_data)
+_mplot = (ui.get_model_plot_prefs, ui.get_model_plot, ui.plot_model)
 
 
 @requires_plotting
-@pytest.mark.parametrize("getprefs,attr,plotfunc",
+@pytest.mark.parametrize("getprefs,getplot,plotfunc",
                          [_dplot, _mplot])
-def test_prefs_change_session_objects(getprefs, attr, plotfunc, clean_ui):
+def test_prefs_change_session_objects(getprefs, getplot, plotfunc, clean_ui):
     """Is a plot-preference change also reflected in the session object?
 
     This is intended to test an assumption that will be used in the
     plot_fit_xxx routines rather than of an explicit user-visible
     behavior. The test may be "obvious behavior" given how
     get_data_plot_prefs works, but DJB wanted to ensure this
-    behavior/assumption was tested.
+    behavior/assumption was tested. The setup has changed since
+    the test was written.
     """
 
-    # This has to be retrieved here, rather than passed in in the
-    # parametrize list, as the ui._session object is changed by
-    # the clean_ui fixture.
+    # This used to access the plot object directly, but is now
+    # using an accessor function.
     #
-    session = getattr(ui._session, attr)
+    session = getplot(recalc=False)
 
     # All but the last assert are just to check things are behaving
     # as expected (and stuck into one routine rather than have a
@@ -781,13 +774,13 @@ def test_prefs_change_session_objects(getprefs, attr, plotfunc, clean_ui):
 def test_prefs_change_session_objects_fit(clean_ui):
     """Is plot-preference change reflected in the fitplot session object?
 
-    This is test_prefs_change_session_objects but for the _fitplot
-    attribute. This test encodes the current behavior - so we can see
+    This is test_prefs_change_session_objects but for the fit plot
+    object. This test encodes the current behavior - so we can see
     if things change in the future - rather than being a statement
     about what we expect/want to happen.
     """
 
-    plotobj = ui._session._fitplot
+    plotobj = ui.get_fit_plot(recalc=False)
     assert plotobj.dataplot is None
     assert plotobj.modelplot is None
 
@@ -808,19 +801,19 @@ def test_prefs_change_session_objects_fit(clean_ui):
     # We have already checked this in previous tests, but
     # just in case
     #
-    assert ui._session._dataplot.plot_prefs['xlog']
-    assert ui._session._modelplot.plot_prefs['ylog']
+    assert ui.get_data_plot(id=12, recalc=False).plot_prefs['xlog']
+    assert ui.get_model_plot(id=12, recalc=False).plot_prefs['ylog']
 
     # Now check that the fit plot has picked up these changes;
     # the simplest way is to check that the data/model plots
     # are now referencing the underlying _data/_model plot
     # attributes. An alternative would be to check that
     # plotobj.dataplot.plot_prefs['xlog'] is True
-    # which is less restrictive, but for not check the
+    # which is less restrictive, but for now check the
     # equality
     #
-    assert plotobj.dataplot is ui._session._dataplot
-    assert plotobj.modelplot is ui._session._modelplot
+    assert plotobj.dataplot is ui.get_data_plot(id=12, recalc=False)
+    assert plotobj.modelplot is ui.get_model_plot(id=12, recalc=False)
 
 
 @pytest.mark.parametrize("plotfunc", [ui.plot_cdf, ui.plot_pdf])
@@ -857,7 +850,7 @@ def test_plot_psf(session):
     s.plot_psf()
 
     plotobj = s.get_psf_plot()
-    assert isinstance(plotobj, PSFPlot)
+    assert isinstance(plotobj, sherpa.plot.PSFPlot)
     assert plotobj.title == 'gauss1d.psfmdl'
     assert plotobj.xlabel == 'x'
     assert plotobj.ylabel == 'y'
@@ -941,7 +934,7 @@ def test_plot_kernel(session, caplog):
     assert logmsg == 'PSF frac: 1.0'
 
     plotobj = s.get_kernel_plot()
-    assert isinstance(plotobj, PSFKernelPlot)
+    assert isinstance(plotobj, sherpa.plot.PSFKernelPlot)
     assert plotobj.title == 'PSF Kernel'
     assert plotobj.xlabel == 'x'
     assert plotobj.ylabel == 'y'
@@ -998,23 +991,18 @@ def test_get_kernel_plot_recalc(session):
 
 
 @requires_plotting
-@pytest.mark.parametrize("plotobj,plotfunc",
-                         [("_residplot", ui.plot_resid),
-                          ("_delchiplot", ui.plot_delchi)
+@pytest.mark.parametrize("getplot,plotfunc",
+                         [(ui.get_resid_plot, ui.plot_resid),
+                          (ui.get_delchi_plot, ui.plot_delchi)
                           ])
-def test_plot_resid_ignores_ylog(plotobj, plotfunc, clean_ui):
+def test_plot_resid_ignores_ylog(getplot, plotfunc, clean_ui):
     """Do the plot_resid-family of routines ignore the ylog setting?
 
     Note that plot_chisqr is not included in support for ignoring
     ylog (since the data should be positive in this case).
     """
 
-    # access it this way to ensure have access to the actual
-    # object used by the session object in this test (as the
-    # clean call done by the clean_ui fixture will reset the
-    # plot objects).
-    #
-    prefs = getattr(ui._session, plotobj).plot_prefs
+    prefs = getplot(recalc=False).plot_prefs
 
     setup_example(None)
 
@@ -1030,20 +1018,15 @@ def test_plot_resid_ignores_ylog(plotobj, plotfunc, clean_ui):
 
 
 @requires_plotting
-@pytest.mark.parametrize("plotobj,plotfunc",
-                         [("_residplot", ui.plot_fit_resid),
-                          ("_delchiplot", ui.plot_fit_delchi)
+@pytest.mark.parametrize("getplot,plotfunc",
+                         [(ui.get_resid_plot, ui.plot_fit_resid),
+                          (ui.get_delchi_plot, ui.plot_fit_delchi)
                           ])
-def test_plot_fit_resid_ignores_ylog(plotobj, plotfunc, clean_ui):
+def test_plot_fit_resid_ignores_ylog(getplot, plotfunc, clean_ui):
     """Do the plot_resid-family of routines ignore the ylog setting?"""
 
-    # access it this way to ensure have access to the actual
-    # object used by the session object in this test (as the
-    # clean call done by the clean_ui fixture will reset the
-    # plot objects).
-    #
-    rprefs = getattr(ui._session, plotobj).plot_prefs
-    dprefs = ui._session._dataplot.plot_prefs
+    rprefs = getplot(recalc=False).plot_prefs
+    dprefs = ui.get_data_plot(recalc=False).plot_prefs
 
     setup_example(None)
 
@@ -1071,12 +1054,10 @@ def test_plot_contour_error_out_invalid(plottype, session):
 
     s = session()
     func = getattr(s, plottype)
-    with pytest.raises(ArgumentErr) as exc:
+    with pytest.raises(PlotErr,
+                       match="Plot type 'fooflan flim flam' not found in"):
         func("fooflan flim flam")
 
-    emsg = "'fooflan flim flam' is not a valid plot type"
-    assert str(exc.value) == emsg
-
 
 @requires_pylab
 @pytest.mark.parametrize("session", [BaseSession, AstroSession])
@@ -1290,12 +1271,12 @@ def test_contour_multiple(session):
 @requires_pylab
 @pytest.mark.parametrize("session", [BaseSession, AstroSession])
 @pytest.mark.parametrize("plotfunc,title,pcls",
-                         [("data", "", DataContour),
-                          ("model", "Model", ModelContour),
-                          ("source", "Source", SourceContour),
-                          ("resid", "Residuals", ResidContour),
-                          ("ratio", "Ratio of Data to Model", RatioContour),
-                          ("fit", "", FitContour),
+                         [("data", "", sherpa.plot.DataContour),
+                          ("model", "Model", sherpa.plot.ModelContour),
+                          ("source", "Source", sherpa.plot.SourceContour),
+                          ("resid", "Residuals", sherpa.plot.ResidContour),
+                          ("ratio", "Ratio of Data to Model", sherpa.plot.RatioContour),
+                          ("fit", "", sherpa.plot.FitContour),
                           ("fit_resid", None, None)])
 def test_contour_xxx(plotfunc, title, pcls, session):
     """Check we can call contour_xxx()/get_xxx_contour().
@@ -1383,7 +1364,7 @@ def test_get_scatter_plot_empty(session):
 
     s = session()
     p = s.get_scatter_plot()
-    assert isinstance(p, ScatterPlot)
+    assert isinstance(p, sherpa.plot.ScatterPlot)
     for f in ['x', 'y', 'xerr', 'yerr', 'xlabel', 'ylabel', 'title']:
         assert getattr(p, f) is None
 
@@ -1403,7 +1384,7 @@ def test_get_scatter_plot(session):
     s.plot_scatter(x, y)
 
     p = s.get_scatter_plot()
-    assert isinstance(p, ScatterPlot)
+    assert isinstance(p, sherpa.plot.ScatterPlot)
 
     assert p.x == pytest.approx(x)
     assert p.y == pytest.approx(y)
@@ -1429,7 +1410,7 @@ def test_get_scatter_plot_labels_noname(session):
     s.plot_scatter(x, y, xlabel='a x', ylabel='43')
 
     p = s.get_scatter_plot()
-    assert isinstance(p, ScatterPlot)
+    assert isinstance(p, sherpa.plot.ScatterPlot)
 
     assert p.x == pytest.approx(x)
     assert p.y == pytest.approx(y)
@@ -1455,7 +1436,7 @@ def test_get_scatter_plot_labels(session):
     s.plot_scatter(x, y, xlabel='a x', ylabel='43', name='Fred Fred')
 
     p = s.get_scatter_plot()
-    assert isinstance(p, ScatterPlot)
+    assert isinstance(p, sherpa.plot.ScatterPlot)
 
     assert p.x == pytest.approx(x)
     assert p.y == pytest.approx(y)
@@ -1475,7 +1456,7 @@ def test_get_trace_plot_empty(session):
 
     s = session()
     p = s.get_trace_plot()
-    assert isinstance(p, TracePlot)
+    assert isinstance(p, sherpa.plot.TracePlot)
     for f in ['x', 'y', 'xerr', 'yerr', 'xlabel', 'ylabel', 'title']:
         assert getattr(p, f) is None
 
@@ -1494,7 +1475,7 @@ def test_get_trace_plot(session):
     s.plot_trace(y)
 
     p = s.get_trace_plot()
-    assert isinstance(p, TracePlot)
+    assert isinstance(p, sherpa.plot.TracePlot)
 
     assert p.x == pytest.approx([0, 1, 2])
     assert p.y == pytest.approx(y)
@@ -1521,7 +1502,7 @@ def test_get_trace_plot_labels_noname(session):
     s.plot_trace(y, xlabel='stats')
 
     p = s.get_trace_plot()
-    assert isinstance(p, TracePlot)
+    assert isinstance(p, sherpa.plot.TracePlot)
 
     assert p.x == pytest.approx([0, 1, 2])
     assert p.y == pytest.approx(y)
@@ -1549,7 +1530,7 @@ def test_get_trace_plot_labels(session):
     s.plot_trace(y, xlabel='stats', name='Awesome sauce')
 
     p = s.get_trace_plot()
-    assert isinstance(p, TracePlot)
+    assert isinstance(p, sherpa.plot.TracePlot)
 
     assert p.x == pytest.approx([0, 1, 2])
     assert p.y == pytest.approx(y)
@@ -1565,7 +1546,7 @@ def test_get_cdf_plot_empty(session):
 
     s = session()
     p = s.get_cdf_plot()
-    assert isinstance(p, CDFPlot)
+    assert isinstance(p, sherpa.plot.CDFPlot)
     for f in ["points", "x", "y", "median", "lower", "upper", "xlabel", "ylabel", "title"]:
         assert getattr(p, f) is None
 
@@ -1631,7 +1612,7 @@ def test_plot_cdf(session):
     s.plot_cdf(x)
 
     p = s.get_cdf_plot()
-    assert isinstance(p, CDFPlot)
+    assert isinstance(p, sherpa.plot.CDFPlot)
 
     xsort = np.sort(x)
     assert p.points == pytest.approx(x)
@@ -1655,7 +1636,7 @@ def test_plot_cdf_labels_noname(session):
     s.plot_cdf(x, xlabel='a b')
 
     p = s.get_cdf_plot()
-    assert isinstance(p, CDFPlot)
+    assert isinstance(p, sherpa.plot.CDFPlot)
 
     assert p.xlabel == 'a b'
     assert p.ylabel == 'p(<=a b)'
@@ -1672,7 +1653,7 @@ def test_plot_cdf_labels(session):
     s.plot_cdf(x, xlabel='a b', name='b a')
 
     p = s.get_cdf_plot()
-    assert isinstance(p, CDFPlot)
+    assert isinstance(p, sherpa.plot.CDFPlot)
 
     assert p.xlabel == 'a b'
     assert p.ylabel == 'p(<=a b)'
@@ -1686,7 +1667,7 @@ def test_show_cdf_plot_empty(session):
     s = session()
 
     p = s.get_cdf_plot()
-    assert isinstance(p, CDFPlot)
+    assert isinstance(p, sherpa.plot.CDFPlot)
 
     toks = str(p).split('\n')
     assert len(toks) == 10
@@ -1721,7 +1702,7 @@ def test_show_cdf_plot(session, use_numpy, old_numpy_printing):
     s.plot_cdf(x)
 
     p = s.get_cdf_plot()
-    assert isinstance(p, CDFPlot)
+    assert isinstance(p, sherpa.plot.CDFPlot)
 
     toks = str(p).split('\n')
     assert len(toks) == 10
@@ -1742,7 +1723,7 @@ def test_get_pdf_plot_empty(session):
 
     s = session()
     p = s.get_pdf_plot()
-    assert isinstance(p, PDFPlot)
+    assert isinstance(p, sherpa.plot.PDFPlot)
     for f in ["points", "xlo", "xhi", "y", "xlabel", "ylabel", "title"]:
         assert getattr(p, f) is None
 
@@ -1757,7 +1738,7 @@ def test_plot_pdf(session):
     s.plot_pdf(x)
 
     p = s.get_pdf_plot()
-    assert isinstance(p, PDFPlot)
+    assert isinstance(p, sherpa.plot.PDFPlot)
 
     xgrid = np.arange(2, 8.5, 0.5)
 
@@ -1852,7 +1833,7 @@ def test_plot_pdf_labels_noname(session):
     s.plot_pdf(x, xlabel='x^2 x', bins=4)
 
     p = s.get_pdf_plot()
-    assert isinstance(p, PDFPlot)
+    assert isinstance(p, sherpa.plot.PDFPlot)
 
     assert p.xlo.size == 4
     assert p.xhi.size == 4
@@ -1873,7 +1854,7 @@ def test_plot_pdf_labels(session):
     s.plot_pdf(x, xlabel='x^2 x', name='no name', bins=4)
 
     p = s.get_pdf_plot()
-    assert isinstance(p, PDFPlot)
+    assert isinstance(p, sherpa.plot.PDFPlot)
 
     assert p.xlo.size == 4
     assert p.xhi.size == 4
@@ -1891,7 +1872,7 @@ def test_show_pdf_plot_empty(session):
     s = session()
 
     p = s.get_pdf_plot()
-    assert isinstance(p, PDFPlot)
+    assert isinstance(p, sherpa.plot.PDFPlot)
 
     toks = str(p).split('\n')
     assert len(toks) == 8
@@ -1920,7 +1901,7 @@ def test_show_pdf_plot(session, old_numpy_printing):
     s.plot_pdf(x, bins=3, normed=False)
 
     p = s.get_pdf_plot()
-    assert isinstance(p, PDFPlot)
+    assert isinstance(p, sherpa.plot.PDFPlot)
 
     toks = str(p).split('\n')
     assert len(toks) == 8
@@ -1946,12 +1927,12 @@ def test_data_plot_recalc(session):
     s.load_arrays(1, [20, 30, 40], [25, 40, 60], [10, 12, 14], Data1DInt)
 
     p = s.get_data_plot(recalc=False)
-    assert isinstance(p, DataHistogramPlot)
+    assert isinstance(p, sherpa.plot.DataHistogramPlot)
     assert p.xlo is None
     assert p.y is None
 
     p = s.get_data_plot(recalc=True)
-    assert isinstance(p, DataHistogramPlot)
+    assert isinstance(p, sherpa.plot.DataHistogramPlot)
     assert p.xlo == pytest.approx([20, 30, 40])
     assert p.xhi == pytest.approx([25, 40, 60])
     assert p.y == pytest.approx([10, 12, 14])
@@ -2026,13 +2007,13 @@ def test_model_plot_recalc(ptype, extraargs, session):
     # just use the ModelPlot.
     #
     p = func(*extraargs, recalc=False)
-    assert isinstance(p, ModelHistogramPlot)
+    assert isinstance(p, sherpa.plot.ModelHistogramPlot)
     assert p.xlo is None
     assert p.xhi is None
     assert p.y is None
 
     p = func(*extraargs, recalc=True)
-    assert isinstance(p, ModelHistogramPlot)
+    assert isinstance(p, sherpa.plot.ModelHistogramPlot)
     assert p.xlo == pytest.approx([20, 30, 40])
     assert p.xhi == pytest.approx([25, 40, 60])
     assert p.y == pytest.approx([162.5, 450, 1200])
@@ -2040,10 +2021,10 @@ def test_model_plot_recalc(ptype, extraargs, session):
 
 @pytest.mark.parametrize("session", [BaseSession, AstroSession])
 @pytest.mark.parametrize("ptype,pclass,y1,y2",
-                         [('resid', ResidPlot, [-10, -12], [-20, -28, -36]),
-                          ('ratio', RatioPlot, [1/11, 0], [1/3, 12/40, 14/50]),
-                          ('delchi', DelchiPlot, [-5, -6], [-10, -14, -18]),
-                          ('chisqr', ChisqrPlot, [25, 36], [100, 14*14, 18*18]),
+                         [('resid', sherpa.plot.ResidPlot, [-10, -12], [-20, -28, -36]),
+                          ('ratio', sherpa.plot.RatioPlot, [1/11, 0], [1/3, 12/40, 14/50]),
+                          ('delchi', sherpa.plot.DelchiPlot, [-5, -6], [-10, -14, -18]),
+                          ('chisqr', sherpa.plot.ChisqrPlot, [25, 36], [100, 14*14, 18*18]),
                           ])
 def test_xxx_plot_recalc(ptype, pclass, y1, y2, session):
     """Basic testing of get_xxx_plot(recalc=False)
@@ -2102,7 +2083,7 @@ def test_fit_plot_recalc(session):
     s.set_source(mdl)
 
     p = s.get_fit_plot(recalc=False)
-    assert isinstance(p, FitPlot)
+    assert isinstance(p, sherpa.plot.FitPlot)
 
     assert p.dataplot.x == pytest.approx([1, 2])
     assert p.dataplot.y == pytest.approx([1, 0])
@@ -2111,7 +2092,7 @@ def test_fit_plot_recalc(session):
     assert p.modelplot.y == pytest.approx([11, 12])
 
     p = s.get_fit_plot(recalc=True)
-    assert isinstance(p, FitPlot)
+    assert isinstance(p, sherpa.plot.FitPlot)
 
     assert p.dataplot.x == pytest.approx([20, 30, 40])
     assert p.dataplot.y == pytest.approx([10, 12, 14])
@@ -2141,7 +2122,7 @@ def check_pvalue(caplog, plot):
     assert toks[4].startswith('p-value          =  0.') or \
         toks[4].startswith('p-value          <  0.01')
 
-    assert isinstance(plot, LRHistogram)
+    assert isinstance(plot, sherpa.plot.LRHistogram)
     assert plot.xlabel == 'Likelihood Ratio'
     assert plot.ylabel == 'Frequency'
     assert plot.title == 'Likelihood Ratio Distribution'
@@ -2183,6 +2164,33 @@ def test_get_pvalue_plot(session, caplog):
     check_pvalue(caplog, p)
 
 
+@pytest.mark.parametrize("session", [BaseSession, AstroSession])
+def test_plot_pvalue_requires_null_model(session):
+    """We need a null_model argument"""
+
+    s = session()
+    with pytest.raises(TypeError) as te:
+        # We add conv_model (to an invalid value) to avoid having
+        # to set up a dataset
+        s.plot_pvalue(None, None, conv_model=True)
+
+    assert str(te.value) == 'null model cannot be None'
+
+
+@pytest.mark.parametrize("session", [BaseSession, AstroSession])
+def test_plot_pvalue_requires_alt_model(session):
+    """We need a alt_model argument"""
+
+    s = session()
+    with pytest.raises(TypeError) as te:
+        # We add conv_model (to an invalid value) to avoid having
+        # to set up a dataset. Similarly all we need is null_model
+        # to be set, not that it's set properly
+        s.plot_pvalue(False, None, conv_model=True)
+
+    assert str(te.value) == 'alternative model cannot be None'
+
+
 @requires_plotting
 @pytest.mark.parametrize("session", [BaseSession, AstroSession])
 def test_plot_pvalue(session, caplog):
@@ -2219,6 +2227,20 @@ def test_plot_pvalue(session, caplog):
     check_pvalue(caplog, p)
 
 
+@pytest.mark.parametrize("session", [BaseSession, AstroSession])
+def test_get_split_plot(session):
+    """Check we can call get_split_plot.
+
+    Limited checking of the result.
+    """
+
+    s = session()
+    splot = s.get_split_plot()
+    assert isinstance(splot, sherpa.plot.SplitPlot)
+    assert splot.rows == 2
+    assert splot.cols == 1
+
+
 @pytest.mark.parametrize("session", [BaseSession, AstroSession])
 def test_get_model_component_plot_invalid(session):
     """invalid model argument for get_model_component_plot"""
@@ -2425,13 +2447,13 @@ def test_data_contour_recalc(session):
     s.load_arrays(1, nx0, nx1, ny, Data2D)
 
     p = s.get_data_contour(recalc=False)
-    assert isinstance(p, DataContour)
+    assert isinstance(p, sherpa.plot.DataContour)
     assert p.x0 == pytest.approx(x0)
     assert p.x1 == pytest.approx(x1)
     assert p.y == pytest.approx(y)
 
     p = s.get_data_contour(recalc=True)
-    assert isinstance(p, DataContour)
+    assert isinstance(p, sherpa.plot.DataContour)
     assert p.x0 == pytest.approx(nx0)
     assert p.x1 == pytest.approx(nx1)
     assert p.y == pytest.approx(ny)
@@ -2469,14 +2491,14 @@ def test_model_contour_recalc(session):
     s.set_source('cmdl')
 
     p = s.get_model_contour(recalc=False)
-    assert isinstance(p, ModelContour)
+    assert isinstance(p, sherpa.plot.ModelContour)
     assert p.x0 == pytest.approx(x0)
     assert p.x1 == pytest.approx(x1)
     # just check the model isn't flat
     assert p.y.min() < p.y.max()
 
     p = s.get_model_contour(recalc=True)
-    assert isinstance(p, ModelContour)
+    assert isinstance(p, sherpa.plot.ModelContour)
     assert p.x0 == pytest.approx(nx0)
     assert p.x1 == pytest.approx(nx1)
     # just check the model is flat
@@ -2515,14 +2537,14 @@ def test_source_contour_recalc(session):
     s.set_source('cmdl')
 
     p = s.get_source_contour(recalc=False)
-    assert isinstance(p, ModelContour)
+    assert isinstance(p, sherpa.plot.ModelContour)
     assert p.x0 == pytest.approx(x0)
     assert p.x1 == pytest.approx(x1)
     # just check the model isn't flat
     assert p.y.min() < p.y.max()
 
     p = s.get_source_contour(recalc=True)
-    assert isinstance(p, ModelContour)
+    assert isinstance(p, sherpa.plot.ModelContour)
     assert p.x0 == pytest.approx(nx0)
     assert p.x1 == pytest.approx(nx1)
     # just check the model is flat
@@ -2561,14 +2583,14 @@ def test_ratio_contour_recalc(session):
     s.set_source('cmdl')
 
     p = s.get_ratio_contour(recalc=False)
-    assert isinstance(p, ModelContour)
+    assert isinstance(p, sherpa.plot.ModelContour)
     assert p.x0 == pytest.approx(x0)
     assert p.x1 == pytest.approx(x1)
     # just check the model isn't flat
     assert p.y.min() < p.y.max()
 
     p = s.get_ratio_contour(recalc=True)
-    assert isinstance(p, ModelContour)
+    assert isinstance(p, sherpa.plot.ModelContour)
     assert p.x0 == pytest.approx(nx0)
     assert p.x1 == pytest.approx(nx1)
 
@@ -2609,14 +2631,14 @@ def test_resid_contour_recalc(session):
     s.set_source('cmdl')
 
     p = s.get_resid_contour(recalc=False)
-    assert isinstance(p, ModelContour)
+    assert isinstance(p, sherpa.plot.ModelContour)
     assert p.x0 == pytest.approx(x0)
     assert p.x1 == pytest.approx(x1)
     # just check the model isn't flat
     assert p.y.min() < p.y.max()
 
     p = s.get_resid_contour(recalc=True)
-    assert isinstance(p, ModelContour)
+    assert isinstance(p, sherpa.plot.ModelContour)
     assert p.x0 == pytest.approx(nx0)
     assert p.x1 == pytest.approx(nx1)
 
@@ -2657,11 +2679,11 @@ def test_fit_contour_recalc(session):
     s.set_source('cmdl')
 
     p = s.get_fit_contour(recalc=False)
-    assert isinstance(p, FitContour)
+    assert isinstance(p, sherpa.plot.FitContour)
     pd = p.datacontour
     pm = p.modelcontour
-    assert isinstance(pd, DataContour)
-    assert isinstance(pm, ModelContour)
+    assert isinstance(pd, sherpa.plot.DataContour)
+    assert isinstance(pm, sherpa.plot.ModelContour)
     assert pd.x0 == pytest.approx(x0)
     assert pd.x1 == pytest.approx(x1)
     assert pm.x0 == pytest.approx(x0)
@@ -2671,11 +2693,11 @@ def test_fit_contour_recalc(session):
     assert pm.y.min() < pm.y.max()
 
     p = s.get_fit_contour(recalc=True)
-    assert isinstance(p, FitContour)
+    assert isinstance(p, sherpa.plot.FitContour)
     pd = p.datacontour
     pm = p.modelcontour
-    assert isinstance(pd, DataContour)
-    assert isinstance(pm, ModelContour)
+    assert isinstance(pd, sherpa.plot.DataContour)
+    assert isinstance(pm, sherpa.plot.ModelContour)
     assert pd.x0 == pytest.approx(nx0)
     assert pd.x1 == pytest.approx(nx1)
     assert pm.x0 == pytest.approx(nx0)
@@ -2829,3 +2851,44 @@ def test_plot_fit_resid_handles_resid_log(idval, clean_ui):
 
     assert axes[1].xaxis.get_scale() == 'log'
     assert axes[1].yaxis.get_scale() == 'linear'
+
+
+@pytest.mark.xfail
+@pytest.mark.parametrize("session", [BaseSession, AstroSession])
+@pytest.mark.parametrize("label", ["source", "model"])
+@pytest.mark.parametrize("idval", [None, 1, "bob"])
+def test_get_foo_component_plot_recalc(session, label, idval):
+    """get_foo_component_plot(recalc=False) should not need a model.
+
+    At the moment it does, hence the xfail mark. See issue #1512
+    """
+
+    s = session()
+    s._add_model_types(basic)
+
+    plot = getattr(s, f"plot_{label}_component")
+    get = getattr(s, f"get_{label}_component_plot")
+
+    mdl = s.create_model_component('polynom1d', 'mdl')
+    mdl.c0 = 10
+    mdl.c1 = 1
+
+    if idval is None:
+        s.load_arrays(1, [-1, 1, 2], [5, 12, 3])
+        s.set_source(mdl)
+    else:
+        s.load_arrays(idval, [-1, 1, 2], [5, 12, 3])
+        s.set_source(idval, mdl)
+
+    if idval is None:
+        plot(mdl)
+        plotobj = get(recalc=False)
+    else:
+        plot(idval, mdl)
+        plotobj = get(idval, recalc=False)
+
+    # The main check is that the get call does not fail, but add some
+    # basic check (fortunately as there is no response here the source
+    # and model values are the same).
+    #
+    assert plotobj.y == pytest.approx([9, 11, 12])
