@@ -86,11 +86,10 @@ def test_check_ids_not_none(func, lo):
     if the lo argument was a string or not,hence the check.
     """
 
-    with pytest.raises(ArgumentTypeErr) as exc:
+    with pytest.raises(ArgumentTypeErr,
+                       match="'ids' must be an identifier or list of identifiers"):
         func(None, lo)
 
-    assert str(exc.value) == "'ids' must be an identifier or list of identifiers"
-
 
 @pytest.mark.parametrize("func", [ui.notice, ui.ignore])
 @pytest.mark.parametrize("lo,hi", [(1, 5), (1, None), (None, 5), (None, None),
@@ -102,11 +101,10 @@ def test_filter_no_data_is_an_error(func, lo, hi, clean_ui):
     had lead to an error state not being reached.
     """
 
-    with pytest.raises(IdentifierErr) as ie:
+    with pytest.raises(IdentifierErr,
+                       match="No data sets found"):
         func(lo, hi)
 
-    assert str(ie.value) == 'No data sets found'
-
 
 def test_save_filter_data1d(tmp_path, clean_ui):
     """Check save_filter [Data1D]"""
@@ -131,18 +129,16 @@ def test_save_filter_data1d(tmp_path, clean_ui):
 
 
 def test_set_iter_method_type_not_string():
-    with pytest.raises(ArgumentTypeErr) as te:
+    with pytest.raises(ArgumentTypeErr,
+                       match="^'meth' must be a string$"):
         ui.set_iter_method(23)
 
-    assert str(te.value) == "'23' must be a string"
-
 
 def test_set_iter_method_type_not_enumeration():
-    with pytest.raises(TypeError) as te:
+    with pytest.raises(TypeError,
+                       match="^a random string is not an iterative fitting method$"):
         ui.set_iter_method('a random string')
 
-    assert str(te.value) == "a random string is not an iterative fitting method"
-
 
 class NonIterableObject:
     """Something that tuple(..) of will error out on"""
@@ -159,19 +155,17 @@ def test_filter_errors_out_invalid_id(func):
     """
 
     ids = NonIterableObject()
-    with pytest.raises(ArgumentTypeErr) as te:
+    with pytest.raises(ArgumentTypeErr,
+                       match="'ids' must be an identifier or list of identifiers"):
         func(ids)
 
-    assert str(te.value) == "'ids' must be an identifier or list of identifiers"
-
 
 def test_set_model_autoassign_func_type():
 
-    with pytest.raises(ArgumentTypeErr) as te:
+    with pytest.raises(ArgumentTypeErr,
+                       match="'func' must be a function or other callable object"):
         ui.set_model_autoassign_func(23)
 
-    assert str(te.value) == "'func' must be a function or other callable object"
-
 
 class DummyModel(ArithmeticModel):
     pass
@@ -296,11 +290,10 @@ def test_err_estimate_errors_on_frozen(method, clean_ui):
 
     ui.load_arrays(1, [1, 2, 3], [1, 2, 3])
     ui.set_source(ui.polynom1d.mdl)
-    with pytest.raises(ParameterErr) as exc:
+    with pytest.raises(ParameterErr,
+                       match="parameter 'mdl.c1' is frozen"):
         method(mdl.c0, mdl.c1)
 
-    assert str(exc.value) == "parameter 'mdl.c1' is frozen"
-
 
 @pytest.mark.parametrize("method", [ui.conf, ui.covar, ui.proj])
 def test_err_estimate_errors_model_all_frozen(method, clean_ui):
@@ -313,11 +306,10 @@ def test_err_estimate_errors_model_all_frozen(method, clean_ui):
     for par in mdl.pars:
         par.freeze()
 
-    with pytest.raises(ParameterErr) as exc:
+    with pytest.raises(ParameterErr,
+                       match="Model 'polynom1d.mdl' has no thawed parameters"):
         method(mdl)
 
-    assert str(exc.value) == "Model 'polynom1d.mdl' has no thawed parameters"
-
 
 @pytest.mark.parametrize("method", [ui.conf, ui.covar, ui.proj])
 @pytest.mark.parametrize("id", [1, "xx"])
@@ -331,11 +323,10 @@ def test_err_estimate_errors_on_list_argument(method, id, otherids, clean_ui):
 
     """
 
-    with pytest.raises(ArgumentTypeErr) as exc:
+    with pytest.raises(ArgumentTypeErr,
+                       match="identifiers must be integers or strings"):
         method(id, otherids)
 
-    assert str(exc.value) == "identifiers must be integers or strings"
-
 
 def setup_err_estimate_multi_ids(strings=False):
     """Create the environment used in test_err_estimate_xxx tests.
@@ -730,7 +721,8 @@ def test_thaw_alwaysfrozen_parameter(string, clean_ui):
     assert ui.get_num_par_thawed() == 3
     assert ui.get_num_par_frozen() == 1
 
-    with pytest.raises(ParameterErr) as pe:
+    with pytest.raises(ParameterErr,
+                       match="^parameter mdl.ref is always frozen and cannot be thawed$"):
         if string:
             ui.thaw("mdl.ref")
         else:
@@ -740,8 +732,6 @@ def test_thaw_alwaysfrozen_parameter(string, clean_ui):
     assert ui.get_num_par_thawed() == 3
     assert ui.get_num_par_frozen() == 1
 
-    assert str(pe.value) == "parameter mdl.ref is always frozen and cannot be thawed"
-
 
 @pytest.mark.parametrize("string", [True, False])
 def test_freeze_model(string, clean_ui):
@@ -888,14 +878,13 @@ def test_freeze_invalid_arguments(string, clean_ui):
     mdl = ui.create_model_component("logparabola", "mdl")
     ui.set_source(mdl)
 
-    with pytest.raises(ArgumentTypeErr) as ae:
+    with pytest.raises(ArgumentTypeErr,
+                       match="^'par' must be a parameter or model object or expression string$"):
         if string:
             ui.freeze("1")
         else:
             ui.freeze(1)
 
-    assert str(ae.value) == "'par' must be a parameter or model object or expression string"
-
 
 @pytest.mark.parametrize("string", [True, False])
 def test_thaw_invalid_arguments(string, clean_ui):
@@ -904,14 +893,13 @@ def test_thaw_invalid_arguments(string, clean_ui):
     mdl = ui.create_model_component("logparabola", "mdl")
     ui.set_source(mdl)
 
-    with pytest.raises(ArgumentTypeErr) as ae:
+    with pytest.raises(ArgumentTypeErr,
+                       match="^'par' must be a parameter or model object or expression string$"):
         if string:
             ui.thaw("1")
         else:
             ui.thaw(1)
 
-    assert str(ae.value) == "'par' must be a parameter or model object or expression string"
-
 
 def test_set_dep_none(clean_ui):
     """What happens if set_dep is called with None?"""
@@ -955,9 +943,9 @@ def test_set_dep_array_wrong(clean_ui):
     ones = np.ones(4)
     ui.load_arrays(1, x, ones)
 
-    # this does not error out
-    ui.set_dep([2, 4, 5])
-    assert ui.get_dep() == pytest.approx([2, 4, 5])
+    with pytest.raises(DataErr,
+                       match="size mismatch between independent axis and y: 4 vs 3"):
+        ui.set_dep([2, 4, 5])
 
 
 def test_set_staterror_none(clean_ui):
@@ -997,11 +985,10 @@ def test_set_syserror_none(clean_ui):
 
     ui.set_syserror(None)
     assert ui.get_staterror() == pytest.approx(staterror)
-    with pytest.raises(DataErr) as err:
+    with pytest.raises(DataErr,
+                       match="data set '1' does not specify systematic errors"):
         ui.get_syserror()
 
-    assert str(err.value) == "data set '1' does not specify systematic errors"
-
     combo = staterror
     assert ui.get_error() == pytest.approx(combo)
 
@@ -1124,14 +1111,13 @@ def test_set_error_array_wrong(field, clean_ui):
 
     setfunc = getattr(ui, f"set_{field}")
 
-    # this does not error out
-    setfunc(np.asarray([1, 2, 3, 4]))
-
-    getfunc = getattr(ui, f"get_{field}")
-    assert getfunc() == pytest.approx([1, 2, 3, 4])
+    with pytest.raises(DataErr,
+                       match=f"size mismatch between independent axis and {field}: 3 vs 4"):
+        setfunc(np.asarray([1, 2, 3, 4]))
 
 
-def test_set_filter_unmasked(clean_ui):
+@pytest.mark.parametrize("ignore", [False, True])
+def test_set_filter_unmasked(ignore, clean_ui):
     """What happens when we call set_filter to an unfiltered dataset?"""
 
     x = np.asarray([10, 20, 30])
@@ -1141,8 +1127,13 @@ def test_set_filter_unmasked(clean_ui):
     data = ui.get_data()
     assert data.mask
 
-    ui.set_filter(np.asarray([True, False, True]))
-    assert data.mask == pytest.approx([True, False, True])
+    if ignore:
+        expected = [False, True, False]
+    else:
+        expected = [True, False, True]
+
+    ui.set_filter(np.asarray([True, False, True]), ignore=ignore)
+    assert data.mask == pytest.approx(expected)
 
 
 def test_set_filter_unmasked_wrong(clean_ui):
@@ -1154,13 +1145,13 @@ def test_set_filter_unmasked_wrong(clean_ui):
 
     data = ui.get_data()
 
-    with pytest.raises(DataErr) as err:
+    with pytest.raises(DataErr,
+                       match="size mismatch between independent axis and mask: 3 vs 2"):
         ui.set_filter(np.asarray([True, False]))
 
-    assert str(err.value) == "size mismatch between 3 and 2"
 
-
-def test_set_filter_masked(clean_ui):
+@pytest.mark.parametrize("ignore", [False, True])
+def test_set_filter_masked(ignore, clean_ui):
     """What happens when we call set_filter to a filtered dataset?"""
 
     x = np.asarray([10, 20, 30, 40, 50])
@@ -1172,8 +1163,16 @@ def test_set_filter_masked(clean_ui):
     data = ui.get_data()
     assert data.mask == pytest.approx([True, False, False, False, True])
 
-    ui.set_filter(np.asarray([True, False, True, False, False]))
-    assert data.mask == pytest.approx([True, False, True, False, True])
+    # Unlike test_set_filter_masked the two expected values are not
+    # logical inverses, since we need to consider the existing mask.
+    #
+    if ignore:
+        expected = [False, False, False, False, True]
+    else:
+        expected = [True, False, True, False, True]
+
+    ui.set_filter(np.asarray([True, False, True, False, False]), ignore=ignore)
+    assert data.mask == pytest.approx(expected)
 
 
 def test_set_filter_masked_wrong(clean_ui):
@@ -1185,7 +1184,6 @@ def test_set_filter_masked_wrong(clean_ui):
 
     ui.ignore(lo=15, hi=45)
 
-    with pytest.raises(DataErr) as err:
+    with pytest.raises(DataErr,
+                       match="size mismatch between data and filter: 3 vs 2"):
         ui.set_filter(np.asarray([True, False]))
-
-    assert str(err.value) == "size mismatch between 3 and 2"
